package query

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
)

type GatewayInfo struct {
	Timestamp int64  `redis:"timestamp" json:"timestamp"`
	Endpoint  string `redis:"endpoint" json:"endpoint"`
}

func (g GatewayInfo) TimeStamp() int64 {
	return g.Timestamp
}

func (g GatewayInfo) Message() json.RawMessage {
	d, _ := json.Marshal(g)
	return d
}

type gatewayQuery struct {
	rdb     *redis.Client
	service common.ServiceType
}

func getGatewayQuery(rdb *redis.Client) ServiceQuery {
	return gatewayQuery{
		rdb:     rdb,
		service: common.SERVICE_API_GATEWAY,
	}
}

func (g gatewayQuery) ModuleName() string {
	return g.service.String()
}

func (g gatewayQuery) List() ([]string, error) {
	all, err := getAllGw(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	var res []string
	for _, v := range all {
		d, _ := json.Marshal(v)
		res = append(res, string(d))
	}
	return res, nil
}

func (g gatewayQuery) ServiceInfo(serviceid string) (string, error) {
	info, err := getOneGw(g.rdb, g.service, serviceid)
	if err != nil {
		return "", err
	}
	d, err := json.Marshal(info)
	if err != nil {
		return "", err
	}

	return string(d), nil
}

func getAllGw(rdb *redis.Client, serviceType common.ServiceType) ([]GatewayInfo, error) {
	keys, err := rdb.Keys(context.Background(), common.GetServiceKeyPrefix(serviceType)+"*").Result()
	if err != nil {
		return nil, err
	}
	var nmInfos []GatewayInfo
	for _, key := range keys {
		res := rdb.HGetAll(context.Background(), key)
		var info GatewayInfo
		if err := res.Scan(&info); err != nil {
			continue
		}
		nmInfos = append(nmInfos, info)
	}
	return nmInfos, nil
}

func getOneGw(rdb *redis.Client, serviceType common.ServiceType, endpoint string) (GatewayInfo, error) {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(serviceType), endpoint)
	res := rdb.HGetAll(context.Background(), k)
	var info GatewayInfo
	if err := res.Scan(&info); err != nil {
		return GatewayInfo{}, err
	}
	return info, nil
}
