package query

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
)

type BackendInfo struct {
	Timestamp int64  `redis:"timestamp" json:"timestamp"`
	Endpoint  string `redis:"endpoint" json:"endpoint"`
}

func (g BackendInfo) TimeStamp() int64 {
	return g.Timestamp
}

func (g BackendInfo) Message() json.RawMessage {
	d, _ := json.Marshal(g)
	return d
}

type backendQuery struct {
	rdb     *redis.Client
	service common.ServiceType
}

func getBackendQuery(rdb *redis.Client) ServiceQuery {
	return backendQuery{
		rdb:     rdb,
		service: common.SERVICE_BACKEND,
	}
}

func (g backendQuery) ModuleName() string {
	return g.service.String()
}

func (g backendQuery) List() ([]string, error) {
	all, err := getAllBackend(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	var res []string
	for _, v := range all {
		d, _ := json.Marshal(v)
		res = append(res, string(d))
	}

	return res, nil
}

func (g backendQuery) ServiceInfo(serviceid string) (string, error) {
	info, err := getOneBackend(g.rdb, g.service, serviceid)
	if err != nil {
		return "", err
	}
	d, err := json.Marshal(info)
	if err != nil {
		return "", err
	}

	return string(d), nil
}

func getAllBackend(rdb *redis.Client, serviceType common.ServiceType) ([]BackendInfo, error) {
	keys, err := rdb.Keys(context.Background(), common.GetServiceKeyPrefix(serviceType)+"*").Result()
	if err != nil {
		return nil, err
	}
	var nmInfos []BackendInfo
	for _, key := range keys {
		res := rdb.HGetAll(context.Background(), key)
		var info BackendInfo
		if err := res.Scan(&info); err != nil {
			continue
		}
		nmInfos = append(nmInfos, info)
	}
	return nmInfos, nil
}

func getOneBackend(rdb *redis.Client, serviceType common.ServiceType, endpoint string) (BackendInfo, error) {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(serviceType), endpoint)
	res := rdb.HGetAll(context.Background(), k)
	var info BackendInfo
	if err := res.Scan(&info); err != nil {
		return BackendInfo{}, err
	}
	return info, nil
}
