package query

import (
	"github.com/odysseus/service-registry/common"
	"github.com/odysseus/service-registry/registry"
	"github.com/redis/go-redis/v9"
)

type ServiceQuerier struct {
	rdb              *redis.Client
	nmQuery          ServiceQuery
	schedulerQuery   ServiceQuery
	gatewayQuery     ServiceQuery
	backendInfoQuery ServiceQuery
	workerQuery      ServiceQuery
}

func NewQuery(redisParam registry.RedisConnParam) *ServiceQuerier {
	rdb := redis.NewClient(&redis.Options{
		Addr:     redisParam.Addr,
		Password: redisParam.Password,
		DB:       redisParam.DbIndex,
	})

	return &ServiceQuerier{
		rdb:              rdb,
		nmQuery:          getNodeManagerQuery(rdb),
		schedulerQuery:   getSchedulerQuery(rdb),
		gatewayQuery:     getGatewayQuery(rdb),
		backendInfoQuery: getBackendQuery(rdb),
		workerQuery:      getWorkerQuery(rdb),
	}
}

func (s *ServiceQuerier) Select(service common.ServiceType) ServiceQuery {
	switch service {
	case common.SERVICE_NODE_MANAGER:
		return s.nmQuery
	case common.SERVICE_SCHEDULER:
		return s.schedulerQuery
	case common.SERVICE_BACKEND:
		return s.backendInfoQuery
	case common.SERVICE_API_GATEWAY:
		return s.gatewayQuery
	case common.SERVICE_WORKER:
		return s.workerQuery
	default:
		return nil
	}

}
