package query

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
)

type SchedulerInfo struct {
	Timestamp int64  `redis:"timestamp" json:"timestamp"`
	Endpoint  string `redis:"endpoint" json:"endpoint"`
}

func (g SchedulerInfo) TimeStamp() int64 {
	return g.Timestamp
}

func (g SchedulerInfo) Message() json.RawMessage {
	d, _ := json.Marshal(g)
	return d
}

type schedulerQuery struct {
	rdb     *redis.Client
	service common.ServiceType
}

func getSchedulerQuery(rdb *redis.Client) ServiceQuery {
	return schedulerQuery{
		rdb:     rdb,
		service: common.SERVICE_SCHEDULER,
	}
}

func (g schedulerQuery) ModuleName() string {
	return g.service.String()
}

func (g schedulerQuery) List() ([]string, error) {
	all, err := getAllSc(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	var res []string
	for _, v := range all {
		d, _ := json.Marshal(v)
		res = append(res, string(d))
	}
	return res, nil
}

func (g schedulerQuery) ServiceInfo(serviceid string) (string, error) {
	info, err := getOneSc(g.rdb, g.service, serviceid)
	if err != nil {
		return "", err
	}
	d, err := json.Marshal(info)
	if err != nil {
		return "", err
	}

	return string(d), nil
}

func getAllSc(rdb *redis.Client, serviceType common.ServiceType) ([]SchedulerInfo, error) {
	keys, err := rdb.Keys(context.Background(), common.GetServiceKeyPrefix(serviceType)+"*").Result()
	if err != nil {
		return nil, err
	}
	var nmInfos []SchedulerInfo
	for _, key := range keys {
		res := rdb.HGetAll(context.Background(), key)
		var info SchedulerInfo
		if err := res.Scan(&info); err != nil {
			continue
		}
		nmInfos = append(nmInfos, info)
	}
	return nmInfos, nil
}

func getOneSc(rdb *redis.Client, serviceType common.ServiceType, endpoint string) (SchedulerInfo, error) {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(serviceType), endpoint)
	res := rdb.HGetAll(context.Background(), k)
	var info SchedulerInfo
	if err := res.Scan(&info); err != nil {
		return SchedulerInfo{}, err
	}
	return info, nil
}
