package query

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/redis/go-redis/v9"
)

type WorkerInfo struct {
	Timestamp      int64    `json:"timestamp"`
	HearBeat       int64    `json:"heartbeat"`
	ActiveNM       []string `json:"active_nm"`
	MinerAddress   string   `json:"miner_address"`
	BenefitAddress string   `json:"benefit_address"`
	IPs            []string `json:"ip_list"`
	Status         string   `json:"status"`
}

func (g WorkerInfo) TimeStamp() int64 {
	return g.Timestamp
}

func (g WorkerInfo) Message() json.RawMessage {
	d, _ := json.Marshal(g)
	return d
}

type workerQuery struct {
	rdb     *redis.Client
	service common.ServiceType
}

func getWorkerQuery(rdb *redis.Client) ServiceQuery {
	return workerQuery{
		rdb:     rdb,
		service: common.SERVICE_WORKER,
	}
}

func (g workerQuery) ModuleName() string {
	return g.service.String()
}

func (g workerQuery) List() ([]string, error) {
	all, err := getAllWorker(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	var res []string
	for _, v := range all {
		d, _ := json.Marshal(v)
		res = append(res, string(d))
	}
	return res, nil
}

func (g workerQuery) ServiceInfo(serviceid string) (string, error) {
	info, err := getOneWorker(g.rdb, g.service, serviceid)
	if err != nil {
		return "", err
	}
	d, err := json.Marshal(info)
	if err != nil {
		return "", err
	}

	return string(d), nil
}

func getAllWorker(rdb *redis.Client, serviceType common.ServiceType) ([]WorkerInfo, error) {
	keys, err := rdb.Keys(context.Background(), common.GetServiceKeyPrefix(serviceType)+"*").Result()
	if err != nil {
		return nil, err
	}
	var nmInfos []WorkerInfo
	for _, key := range keys {
		res := rdb.HGetAll(context.Background(), key)
		var info WorkerInfo
		if err := res.Scan(&info); err != nil {
			continue
		}
		nmInfos = append(nmInfos, info)
	}
	return nmInfos, nil
}

func getOneWorker(rdb *redis.Client, serviceType common.ServiceType, endpoint string) (WorkerInfo, error) {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(serviceType), endpoint)
	res := rdb.HGetAll(context.Background(), k)
	var info WorkerInfo
	if err := res.Scan(&info); err != nil {
		return WorkerInfo{}, err
	}
	return info, nil
}
